import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleBand, scaleLinear, max } from 'd3';

const width = window.innerWidth;
const height = window.innerHeight;
const margin = { top: 20, right: 30, left: 80, bottom: 110 };
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

const yTickOffset = 5;

const csvUrl =
  'https://gist.githubusercontent.com/minwoopak/07eb41dfcd831afea0847ac77d1d09f5/raw/UN_Population_2019.csv';

const App = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    const row = (d) => {
      d.Population = +d['2020'];
      return d;
    };
    csv(csvUrl, row).then((data) => {
      data = data.slice(0, 10);
      setData(data);
    });
  }, []);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const xScale = scaleBand()
    .domain(data.map((d) => d.Country))
    .range([0, innerWidth])
  	.padding(0.2);

  const yScale = scaleLinear()
    .domain([0, max(data, (d) => d.Population)])
    .range([innerHeight, 0]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        // Y-Axis
        {yScale.ticks().map((tickValue) => (
          <g
            key={tickValue}
            transform={`translate(${0}, ${yScale(
              tickValue
            )})`}
          >
            <line x2={innerWidth} stroke="black" />
            <text
              style={{ textAnchor: 'end' }}
              x={-5}
              dy=".32em"
            >
              {tickValue}
            </text>
          </g>
        ))}
        // X-Axis
        {xScale.domain().map((tickValue) => (
          <text
            key={tickValue}
            transform={`rotate(${30})`}
            style={{
              textAnchor: 'start',
              transformOrigin: `${
                xScale(tickValue) +
                (xScale.bandwidth() * 1) / 2
              }px ${innerHeight + yTickOffset}px`,
            }}
            x={
              xScale(tickValue) +
              (xScale.bandwidth() * 1) / 2
            }
            y={innerHeight + yTickOffset}
            dy={'.71em'}
          >
            {tickValue}
          </text>
        ))}
        // Bars
        {data.map((d) => (
          <rect
            key={d.Country}
            x={xScale(d.Country)}
            y={yScale(d.Population)}
            width={xScale.bandwidth()}
            height={innerHeight - yScale(d.Population)}
          />
        ))}
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
